#!/usr/bin/env python
# -*- coding: ISO-8859-1 -*-

# Copyright (C) 2002, 2003 Jrg Lehmann <joerg@luga.de>
#
# This file is part of PyTone (http://www.luga.de/pytone/)
#
# PyTone is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2
# as published by the Free Software Foundation.
#
# PyTone is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyX; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import config
import songdb
import item
import os, glob

# dummy song class for testing purposes

class song(item.song):
    def __init__(self, path):
        self.path = path
        self.id = path
        self.title, self.album, self.artist, self.year, self.genre = \
                    path.split("/")
        self.tracknr = ""
        self.length = 0
        self.nrplayed = 0
        self.lastplayed = None
        self.rating = None

# replace standard implementation in item module
item.song = song

for dbfile in glob.glob("test_*.db"):
    os.unlink(dbfile)

config.database.basename="test"

def checksongs(csongs):
    songs = db._getsongs()
    assert len(songs)==len(csongs), "Error in song list: wrong size"
    for song in songs:
        assert song.path in csongs, "Error in song list: wrong content"

def checkgenres(cgenres):
    genres = db._getgenres()
    assert len(genres)==len(cgenres), "Error in genre list: wrong size"
    for genre in genres:
        assert genre.name in cgenres, "Error in genre list: wrong content"
        cgenre = cgenres[genre.name]
        assert genre.artists == cgenre[0], "Error in genre list: wrong content"
        assert genre.albums == cgenre[1], "Error in genre list: wrong content"

        assert genre.songs == cgenre[2], "Error in genre list: wrong content"

def checkyears(cyears):
    years = db._getyears()
    assert len(years)==len(cyears), "Error in year list: wrong size"
    for year in years:
        assert year.year in cyears, "Error in year list: wrong content"
        cyear = cyears[year.year]
        assert year.artists == cyear, "Error in year list: wrong content"

def checkalbums(artist, calbums):
    albums = db._getalbums(artist)
    assert len(albums)==len(calbums), "Error in album list for %s: wrong size" % artist
    for album in albums:
        assert album.name in calbums, "Error in year list: wrong content"


##############################################################################
# test insertion and deletion of songs
##############################################################################

db = songdb.songdb()

song1 = "Titel1/Album1/Artist1/1/Genre1"

db._queryregistersong(song1)

checksongs([song1])
checkgenres({"Genre1": (["Artist1"], [item._albumid("Artist1", "Album1")], [song1])})
checkyears({"1": ["Artist1"]})

song2 = "Titel2/Album1/Artist1/1/Genre1"
db._queryregistersong(song2)

checksongs([song1, song2])
checkgenres({"Genre1": (["Artist1"], [item._albumid("Artist1", "Album1")], [song1, song2])})
checkyears({"1": ["Artist1"]})

#
# check different genres and years for same artist
#

song3 = "Titel2/Album1/Artist1/2/Genre2"
db._queryregistersong(song3)

checkgenres({"Genre1": (["Artist1"], [item._albumid("Artist1", "Album1")], [song1, song2]),
             "Genre2": (["Artist1"], [item._albumid("Artist1", "Album1")], [song3])})
checkyears({"1": ["Artist1"], "2": ["Artist1"]})

db._delsong(song3)

#
# check different artists for genres and years
#

song4 = "Titel2/Album1/Artist2/1/Genre1"
db._queryregistersong(song4)

checkgenres({"Genre1": (["Artist1", "Artist2"],
                        [item._albumid("Artist1", "Album1"),
                         item._albumid("Artist2", "Album1")],
                        [song1, song2, song4])})
checkyears({"1": ["Artist1", "Artist2"]})

db._delsong(song4)

checkgenres({"Genre1": (["Artist1"],
                        [item._albumid("Artist1", "Album1")],
                        [song1, song2])})
checkyears({"1": ["Artist1"]})

#
# check different albums for genres and years
#

song5 = "Titel2/Album2/Artist1/1/Genre1"
db._queryregistersong(song5)

checkgenres({"Genre1": (["Artist1"],
                        [item._albumid("Artist1", "Album1"),
                         item._albumid("Artist1", "Album2")],
                        [song1, song2, song5])})

checkyears({"1": ["Artist1"]})
checkalbums("Artist1", ["Album1", "Album2"])

db._delsong(song5)

checksongs([song1, song2])

checkgenres({"Genre1": (["Artist1"],
                        [item._albumid("Artist1", "Album1")],
                        [song1, song2])})
checkyears({"1": ["Artist1"]})
checkalbums("Artist1", ["Album1"])

db._delsong(song1)

checksongs([song2])
checkgenres({"Genre1": (["Artist1"],
                        [item._albumid("Artist1", "Album1")],
                        [song2])})
checkyears({"1": ["Artist1"]})

db._delsong(song2)

checksongs([])
checkgenres({})
checkyears({})


##############################################################################
# test update of songs
##############################################################################

s1 = db._queryregistersong(song1)

s1.artist = "Artist2"
s1.album = "Album2"
s1.genre = "Genre2"
s1.year = "2"
db._updatesong(s1)

checksongs([song1])
checkgenres({"Genre2": (["Artist2"], [item._albumid("Artist2", "Album2")], [s1.id])})
checkyears({"2": ["Artist2"]})
checkalbums("Artist2", ["Album2"])

db.close() 
